@@ -861,6 +861,40 @@ grn_obj_is_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
   }
 }
 
+bool
+grn_obj_is_text_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
+{
+  grn_accessor *accessor;
+
+  if (!grn_obj_is_accessor(ctx, obj)) {
+    return false;
+  }
+
+  accessor = (grn_accessor *)obj;
+  while (accessor->next) {
+    accessor = accessor->next;
+  }
+
+  switch (accessor->action) {
+  case GRN_ACCESSOR_GET_ID:
+  case GRN_ACCESSOR_GET_SCORE:
+  case GRN_ACCESSOR_GET_NSUBRECS:
+  case GRN_ACCESSOR_GET_MAX:
+  case GRN_ACCESSOR_GET_MIN:
+  case GRN_ACCESSOR_GET_SUM:
+  case GRN_ACCESSOR_GET_AVG:
+  case GRN_ACCESSOR_GET_MEAN:
+    return false;
+  case GRN_ACCESSOR_GET_VALUE:
+    return grn_type_id_is_text_family(ctx,
+                                      grn_obj_get_range(ctx, accessor->obj));
+  case GRN_ACCESSOR_GET_COLUMN_VALUE:
+    return grn_obj_is_text_family_scalar_column(ctx, accessor->obj);
+  default:
+    return false;
+  }
+}
+
 bool
 grn_obj_is_number_family_scalar_accessor(grn_ctx *ctx, grn_obj *obj)
 {
@@ -1134,13 +1168,14 @@ grn_obj_set_visibility(grn_ctx *ctx, grn_obj *obj, bool is_visible)
   }
 
   if (!(grn_obj_is_index_column(ctx, obj) ||
-        grn_obj_is_token_column(ctx, obj))) {
+        grn_obj_is_token_column(ctx, obj) ||
+        grn_obj_is_generated_column(ctx, obj))) {
     grn_obj inspected;
     GRN_TEXT_INIT(&inspected, 0);
     grn_inspect_limited(ctx, &inspected, obj);
     ERR(GRN_INVALID_ARGUMENT,
         "[obj][set-visibility] "
-        "must be an index column or a token column: <%.*s>",
+        "must be an index column, a token column or a generated column: <%.*s>",
         (int)(GRN_TEXT_LEN(&inspected)),
         GRN_TEXT_VALUE(&inspected));
     GRN_OBJ_FIN(ctx, &inspected);
@@ -1173,7 +1208,33 @@ grn_obj_have_source(grn_ctx *ctx, grn_obj *obj)
 bool
 grn_obj_is_token_column(grn_ctx *ctx, grn_obj *obj)
 {
-  return grn_obj_is_vector_column(ctx, obj) && grn_obj_have_source(ctx, obj);
+  return grn_obj_is_vector_column(ctx, obj) && grn_obj_have_source(ctx, obj) &&
+         ((grn_ja *)obj)->generator.length == 0;
+}
+
+bool
+grn_obj_is_generated_column(grn_ctx *ctx, grn_obj *obj)
+{
+  if (!obj) {
+    return false;
+  }
+
+  grn_raw_string *generator = NULL;
+  switch (obj->header.type) {
+  case GRN_COLUMN_FIX_SIZE:
+    generator = &(((grn_ra *)obj)->generator);
+    break;
+  case GRN_COLUMN_VAR_SIZE:
+    generator = &(((grn_ja *)obj)->generator);
+    break;
+  default:
+    return false;
+  }
+  if (!grn_obj_have_source(ctx, obj)) {
+    return false;
+  }
+
+  return generator->length > 0;
 }
 
 static void
